﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace CoscomV4_Tutorial
{
    /// <summary>
    /// Interaktionslogik für EventMaskControl.xaml
    /// </summary>
    public partial class EventMaskControl : UserControl, INotifyPropertyChanged
    {
        /// <summary>
        /// Indicating if checkbox changes should be ignored
        /// </summary>
        bool MyIsInitialized = false;

        XDOP.schemas_coscom_org.service.ISBasicControlService MyService = null; // service instance

        #region WPF_Binding
        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string prop)
        {
            if (Dispatcher.Thread == System.Threading.Thread.CurrentThread)
            {
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs(prop));
                }
            }
            else
                Dispatcher.InvokeAsync(() =>
                {
                    if (PropertyChanged != null)
                    {
                        PropertyChanged(this, new PropertyChangedEventArgs(prop));
                    }
                }, System.Windows.Threading.DispatcherPriority.Normal);
        }
        #endregion

        public EventMaskControl()
        {
            InitializeComponent();
            // set checkbox states according to current event mask
            vControlStatus.IsChecked = MyCurrentEventMask[21] == '1';
            vControlAllowed.IsChecked = MyCurrentEventMask[20] == '1';
            vActualSpeed.IsChecked = MyCurrentEventMask[19] == '1';
            vTargetSpeed.IsChecked = MyCurrentEventMask[18] == '1';
            vActualElevation.IsChecked = MyCurrentEventMask[17] == '1';
            vTargetElevation.IsChecked = MyCurrentEventMask[16] == '1';
            vActualPower.IsChecked = MyCurrentEventMask[15] == '1';
            vTargetPower.IsChecked = MyCurrentEventMask[14] == '1';
            vEnergyConsumption.IsChecked = MyCurrentEventMask[13] == '1';
            vMET.IsChecked = MyCurrentEventMask[12] == '1';
            vTime.IsChecked = MyCurrentEventMask[11] == '1';
            vDistance.IsChecked = MyCurrentEventMask[10] == '1';
            vActualCadence.IsChecked = MyCurrentEventMask[9] == '1';
            vHeight.IsChecked = MyCurrentEventMask[8] == '1';
            vHeartRate.IsChecked = MyCurrentEventMask[7] == '1';
            vRRInterval.IsChecked = MyCurrentEventMask[6] == '1';
            vErrors.IsChecked = MyCurrentEventMask[5] == '1';
            // index 17 is not used
            vActualTorque.IsChecked = MyCurrentEventMask[3] == '1';
            vTargetTorque.IsChecked = MyCurrentEventMask[2] == '1';
            vStepHeight.IsChecked = MyCurrentEventMask[1] == '1';
            vTargetCadence.IsChecked = MyCurrentEventMask[0] == '1';
            MyIsInitialized = true;
        }

        /// Current event mask string
        static string MyCurrentEventMask = "1111111111111111111111";

        /// <summary>
        /// Method for getting mask in main window
        /// </summary>
        public static string GetCurrentEventMask
        {
            get
            {
                return MyCurrentEventMask;
            }
        }
        public string CurrentEventMask
        {
            get { return MyCurrentEventMask; }
            set
            {
                // setter for wpf binding
                MyCurrentEventMask = value;
                OnPropertyChanged("CurrentEventMask");
            }
        }
        private void SetMask_Click(object sender, RoutedEventArgs e)
        {
            if (MyService != null)
            {
                try
                {
                    // set a new elevation value
                    // Note do not check actual number in order that one can simulate error messages from device side if parameter is out of range.
                    MyService.SetEventMask(CurrentEventMask);
                }
                catch (Exception er)
                {
                    MyService = null;
                    MessageBox.Show("Error: " + er.Message);
                }
            }
        }
      
        /// <summary>
        /// Set service in order to set event mask if needed
        /// </summary>
        /// <param name="service"></param>
        public void SetService(XDOP.schemas_coscom_org.service.ISBasicControlService service)
        {
            MyService = service;
        }

        /// <summary>
        /// if a checkbox changes value set event mask accordingly
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cB_CheckedChanged(object sender, RoutedEventArgs e)
        {
            if (!MyIsInitialized)
                return;
            string result = "";
            result += vTargetCadence.IsChecked.Value ? "1" : "0";
            result += vStepHeight.IsChecked.Value ? "1" : "0";
            result += vTargetTorque.IsChecked.Value ? "1" : "0";
            result += vActualTorque.IsChecked.Value ? "1" : "0";
            //result += vHeight.IsChecked.Value ? "1" : "0";
            result += "0"; // index 17 is not used, set it to off = 0
            result += vErrors.IsChecked.Value ? "1" : "0";
            result += vRRInterval.IsChecked.Value ? "1" : "0";
            result += vHeartRate.IsChecked.Value ? "1" : "0";
            result += vHeight.IsChecked.Value ? "1" : "0";
            result += vActualCadence.IsChecked.Value ? "1" : "0";
            result += vDistance.IsChecked.Value ? "1" : "0";
            result += vTime.IsChecked.Value ? "1" : "0";
            result += vMET.IsChecked.Value ? "1" : "0";
            result += vEnergyConsumption.IsChecked.Value ? "1" : "0";
            result += vTargetPower.IsChecked.Value ? "1" : "0";
            result += vActualPower.IsChecked.Value ? "1" : "0";
            result += vTargetElevation.IsChecked.Value ? "1" : "0";
            result += vActualElevation.IsChecked.Value ? "1" : "0";
            result += vTargetSpeed.IsChecked.Value ? "1" : "0";
            result += vActualSpeed.IsChecked.Value ? "1" : "0";
            result += vControlAllowed.IsChecked.Value ? "1" : "0";
            result += vControlStatus.IsChecked.Value ? "1" : "0";
            CurrentEventMask = result;
        }

        public event EventHandler Done;
        private void Done_Click(object sender, RoutedEventArgs e)
        {
            Done?.Invoke(this, new EventArgs());
        }
    }
}
